 /*
  * Jake Moilanen <jmoilane@umich.edu>
  * Kendrick Smith <kmsmith@umich.edu>
  * Andy Adamson <andros@umich.edu>
  * Copyright (C) 2001 The Regents of the University of Michigan
  *
  */

#include <linux/sched.h>
#include <linux/smp_lock.h>
#include <linux/mm.h>
#include <linux/pagemap.h>
#include <linux/fs.h>
#include <linux/acl_ea.h>
#include <linux/ext_attr.h>
#include <asm/uaccess.h>

#include <linux/nfs4/nfs4_prot.h>
#include <linux/nfs4/nfs4_debug.h>
#include <linux/nfs4/nfs4.h>
#include <linux/nfs4/nfs4fs.h>

#ifdef NFS4ACL

extern fattr4_acl * posix_acl_to_nfsv4_acl(posix_acl_t * acl);
extern posix_acl_t * nfsv4_acl_to_posix_acl(nfs4_acl * nfs4_acl, int num_acl);


#ifndef SET_ACE
#define SET_ACE(in_type, in_flag, in_mask, in_len, in_val) do {	\
	entries[i].type = in_type;				\
	entries[i].flag = in_flag;				\
	entries[i].access_mask = in_mask;			\
	entries[i].who.utf8string_len = in_len;			\
	entries[i].who.utf8string_val = NFS4_ALLOC(in_len);	\
	memcpy(entries[i].who.utf8string_val, in_val, in_len); \
} while (0)
#endif

#ifndef SET_MASK_POSIX
#define SET_MASK_POSIX(check, mask)	do {			\
	if (check & ACL_READ) 					\
		mask |= ACE4_GENERIC_READ;			\
        if (check & ACL_WRITE)					\
                mask |= ACE4_GENERIC_WRITE;			\
        if (check & ACL_EXECUTE)    				\
                mask |= ACE4_GENERIC_EXECUTE;			\
} while (0)
#endif

#ifndef SET_MASK_NFS4
#define SET_MASK_NFS4(check) do { 				\
        mode = 0;						\
        if ((check & ACE4_GENERIC_READ) == ACE4_GENERIC_READ)	\
		mode |= ACL_READ;				\
        if ((check & ACE4_GENERIC_WRITE) == ACE4_GENERIC_WRITE) \
		mode |= ACL_WRITE;				\
	if ((check & ACE4_GENERIC_EXECUTE) == ACE4_GENERIC_EXECUTE) \
	        mode |= ACL_EXECUTE;				\
} while (0)
#endif

struct posix_acl_list {
	posix_acl_entry_t *pa;
	struct list_head l_acl;
};
typedef struct posix_acl_list posix_acl_list;

struct nfsv4_acl_list {
	nfs4_acl	 *acl;	
	struct list_head l_acl;
};
typedef struct nfsv4_acl_list nfsv4_acl_list;

/*
 * Converts a posix acl to a nfsv4 acl
 *
 * Puts in order:
 *  user owner
 *  user
 *  group owner
 *  group
 *  other
 *
 * XXX Need to do
 * Add the @computer.name to the names
 * make sure Free
 * error check if group owner exists or user owner exists
 */
fattr4_acl *
posix_acl_to_nfsv4_acl(posix_acl_t * acl)
{
	const u32 		deny_mask = 0x001fffff;
	posix_acl_entry_t 	*pa, *pe;
	fattr4_acl 		*ace;
	nfsace4 		*entries;
	int 			i = 0;
	int 			status;
	u32 			mask;
	struct gss_cacheent 	*gss_who;
	struct list_head	user_list;
	struct list_head	group_list;
	struct list_head 	other_list;
	struct posix_acl_list	*tmp_list;
	struct list_head	*p;
	posix_acl_entry_t 	*user_owner_acl = 0;
	posix_acl_entry_t	*group_owner_acl = 0;
	printk("posix_acl_to_nfsv4_acl: I'm in\n");

	/* order and put into the correct lists */
	if (!acl) {
		nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: No acl\n");
		return 0;
	}

	INIT_LIST_HEAD(&user_list);
	INIT_LIST_HEAD(&group_list);
	INIT_LIST_HEAD(&other_list);	
	FOREACH_ACL_ENTRY(pa, acl, pe){
		
		printk("posix_acl_to_nfsv4_acl: -----------------------\n");
		switch(pa->e_tag) {
               		case ACL_USER_OBJ:
				printk("posix_acl_to_nfsv4_acl: ACL_USER_OBJ\n");
				if (user_owner_acl != 0)
					nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: ERROR two owner acls\n");  // XXX what should I do in this case
				user_owner_acl = pa;
                                break;
               	        case ACL_USER:
				printk("posix_acl_to_nfsv4_acl: ACL_USER\n");
				tmp_list = NFS4_ALLOC(sizeof(posix_acl_list));
				tmp_list->pa = pa;
				list_add_tail(&tmp_list->l_acl, &user_list);
                                break;
               	        case ACL_GROUP_OBJ:
				printk("posix_acl_to_nfsv4_acl: ACL_GROUP_OBJ\n");
				if (group_owner_acl != 0)
					nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: ERROR two group owner acls\n");  // XXX what should I do in this case
				group_owner_acl = pa;
                                break;
               	        case ACL_GROUP:
				printk("posix_acl_to_nfsv4_acl: ACL_GROUP\n");
				tmp_list = NFS4_ALLOC(sizeof(posix_acl_list));
				tmp_list->pa = pa;
				list_add_tail(&tmp_list->l_acl, &group_list);
                                break;
               	        case ACL_MASK:
				printk("posix_acl_to_nfsv4_acl: ACL_MASK\n");
                               	break;
                       	case ACL_OTHER:
				printk("posix_acl_to_nfsv4_acl: ACL_OTHER\n");
				tmp_list = NFS4_ALLOC(sizeof(posix_acl_list));
				tmp_list->pa = pa;
				list_add_tail(&tmp_list->l_acl, &other_list);
                       	        break;
	                default:
	 			nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: Unrecognized posix acl type\n");
               	                break;
              	} // switch		
	} // FOREACH_ACL_ENTRY

	/* put into NFSv4 acl */
	ace = NFS4_ALLOC(sizeof(fattr4_acl));

	if (!acl->a_count) {
		printk("posix_acl_to_nfsv4_acl: No acl's\n");
		ace->fattr4_acl_len = 0;
		return ace;
	}

	printk("posix_acl_to_nfsv4_acl: acl->a_count=%d\n", acl->a_count);
	printk("posix_acl_to_nfsv4_acl: size alloc'd=%d\n", (sizeof(nfsace4) * acl->a_count) * 2);
	ace->fattr4_acl_val = NFS4_ALLOC((sizeof(nfsace4) * acl->a_count) * 2); /* This is the max */
	entries = ace->fattr4_acl_val;
	
	/* user owner */
	if (user_owner_acl) {
		mask = 0;
		SET_MASK_POSIX(user_owner_acl->e_perm, mask);
		SET_ACE(ACE4_ACCESS_ALLOWED_ACE_TYPE, 0, mask, 6, "OWNER@");
		i++; 
		SET_ACE(ACE4_ACCESS_DENIED_ACE_TYPE, 0, deny_mask, 6, "OWNER@");
		i++;
	} // if

	/* user acls */
	list_for_each(p, &user_list){ 
		tmp_list = list_entry(p, posix_acl_list, l_acl);
		pa = tmp_list->pa;
		mask = 0;
		if ((status = gss_get_name(GSS_OWNER, pa->e_id, &gss_who))){
			nfsv4_printk(acl, "posix_acl_to_nfsv4_acl: gss_get_name failed\n");
		}
		SET_MASK_POSIX(pa->e_perm, mask);
		SET_ACE(ACE4_ACCESS_ALLOWED_ACE_TYPE, 0, mask, gss_who->name.len, gss_who->name.name);
		i++;
		SET_ACE(ACE4_ACCESS_DENIED_ACE_TYPE, 0, deny_mask, gss_who->name.len, gss_who->name.name);
		i++;
		gss_put(gss_who);
	} // list_for_each

	/* positive acl's first */
	/* group owner */
	if (group_owner_acl){
        	mask = 0;
        	SET_MASK_POSIX(group_owner_acl->e_perm, mask);
        	SET_ACE(ACE4_ACCESS_ALLOWED_ACE_TYPE, ACE4_IDENTIFIER_GROUP, mask, 6, "GROUP@");
        	i++;
	} // if

        /* group acls */
        list_for_each(p, &group_list){
                tmp_list = list_entry(p, posix_acl_list, l_acl);
                pa = tmp_list->pa;
                mask = 0;
                if ((status = gss_get_name(GSS_GROUP, pa->e_id, &gss_who))){
                        nfsv4_printk(acl, "posix_acl_to_nfsv4_acl: gss_get_name failed\n");
                }
                SET_MASK_POSIX(pa->e_perm, mask);
                SET_ACE(ACE4_ACCESS_ALLOWED_ACE_TYPE, ACE4_IDENTIFIER_GROUP, mask, gss_who->name.len, gss_who->name.name);
                i++;
		gss_put(gss_who);
        } // list_for_each

	/* negatvie acls */
	/* group owner */
	if (group_owner_acl) {
        	SET_ACE(ACE4_ACCESS_DENIED_ACE_TYPE, ACE4_IDENTIFIER_GROUP, deny_mask, 6, "GROUP@"); 
		i++; 
	} // if 

	/* group */
        list_for_each(p, &group_list){
                tmp_list = list_entry(p, posix_acl_list, l_acl);
                pa = tmp_list->pa;
                if ((status = gss_get_name(GSS_GROUP, pa->e_id, &gss_who))){
                        nfsv4_printk(acl, "posix_acl_to_nfsv4_acl: gss_get_name failed\n");
                }
                SET_ACE(ACE4_ACCESS_DENIED_ACE_TYPE, ACE4_IDENTIFIER_GROUP, deny_mask, gss_who->name.len, gss_who->name.name);
                i++;
		gss_put(gss_who);
        } // list_for_each

	/* other */
        list_for_each(p, &other_list){
                tmp_list = list_entry(p, posix_acl_list, l_acl);
                pa = tmp_list->pa;
                mask = 0;
                SET_MASK_POSIX(pa->e_perm, mask);
                SET_ACE(ACE4_ACCESS_ALLOWED_ACE_TYPE, 0, mask, 9, "EVERYONE@");
                i++;
        } // list_for_each

	ace->fattr4_acl_len = i;
	printk("posix_acl_to_nfsv4_acl: i (len)=%d\n", i);
	printk("posix_acl_to_nfsv4_acl: About to clean\n");

	/* cleanup */
	while(!list_empty(&user_list)) {
		printk("user_list free\n");
		p = user_list.next;
		list_del(p);
		tmp_list = list_entry(p, posix_acl_list, l_acl);
		NFS4_FREE(tmp_list);
	} // while 		

	while(!list_empty(&group_list)) {
		printk("group_list free\n");
		p = group_list.next;
		list_del(p);
		tmp_list = list_entry(p, posix_acl_list, l_acl);
		NFS4_FREE(tmp_list);
	} // while 		

	while(!list_empty(&other_list)) {
		printk("other_list free \n");
		p = other_list.next;
		list_del(p);
		tmp_list = list_entry(p, posix_acl_list, l_acl);
		NFS4_FREE(tmp_list);
	} // while 		
	printk("posix_acl_to_nfsv4_acl: I'm out\n");
	return ace;
}



posix_acl_t * 
nfsv4_acl_to_posix_acl(nfs4_acl * nfs4_acl, int num_acl)
{
	posix_acl_t 		*acl;
	struct nfs4_acl		*tmp_acl;
	int			num_pos_acl = 0;	// num of positive acls
	u32			mask = -1;		// keep track of mask
	u32			mode = 0;
	struct list_head	user_list;
	struct list_head	group_list;
	struct list_head 	other_list;
	nfsv4_acl_list		*tmp_list;
	struct list_head	*p;
	struct nfs4_acl	 	*user_owner_acl = 0;
	struct nfs4_acl		*group_owner_acl = 0;
	int 			i;

	printk("nfsv4_acl_to_posix_acl: I'm in\n");

	INIT_LIST_HEAD(&user_list);
	INIT_LIST_HEAD(&group_list);
	INIT_LIST_HEAD(&other_list);	

	for (i = 0; i < num_acl; i++){
		if (nfs4_acl[i].type == ACE4_ACCESS_ALLOWED_ACE_TYPE){
		
			if (nfs4_acl[i].flag & ACE4_OWNER){
				printk("nfsv4_acl_to_posix_acl: ACL_USER_OBJ\n"); 
				if (user_owner_acl != 0)
					nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: ERROR two owner acls\n");  // XXX what should I do 
				user_owner_acl = &nfs4_acl[i];
			} else if (nfs4_acl[i].id != -1 && !(nfs4_acl[i].flag & ACE4_IDENTIFIER_GROUP)) {
				printk("nfsv4_acl_to_posix_acl: ACL_USER\n");
				tmp_list = NFS4_ALLOC(sizeof(nfsv4_acl_list));
				tmp_list->acl = &nfs4_acl[i];
				list_add_tail(&tmp_list->l_acl, &user_list);
				mask = 0;
			} else if (nfs4_acl[i].flag & ACE4_GROUP) {
				printk("nfsv4_acl_to_posix_acl: ACL_GROUP_OBJ\n");
				if (group_owner_acl != 0)
					nfsv4_printk(acl2, "posix_acl_to_nfsv4_acl: ERROR two group owner acls\n");  // XXX what should
				group_owner_acl = &nfs4_acl[i];
				mask = 0;
			} else if (nfs4_acl[i].flag & ACE4_IDENTIFIER_GROUP) {
				printk("nfsv4_acl_to_posix_acl: ACL_GROUP\n");
				tmp_list = NFS4_ALLOC(sizeof(nfsv4_acl_list));
				tmp_list->acl = &nfs4_acl[i];
				list_add_tail(&tmp_list->l_acl, &group_list);
				mask = 0;
                        } else if (nfs4_acl[i].flag & ACE4_EVERYONE) {
				printk("nfsv4_acl_to_posix_acl: ACL_OTHER\n");
				tmp_list = NFS4_ALLOC(sizeof(nfsv4_acl_list));
				tmp_list->acl = &nfs4_acl[i];
				list_add_tail(&tmp_list->l_acl, &other_list);
			} else {
	 			nfsv4_printk(acl2, "nfsv4_acl_to_posix_acl: Unrecognized posix acl type\n");
			}
		num_pos_acl++;
		} // if

	} // for
	printk("nfsv4_acl_to_posix_acl: acl alloc\n");
	printk("nfsv4_acl_to_posix_acl: num_pos_acl + !mask ? 1:0=%d\n", num_pos_acl + (!mask ? 1:0));
	acl = posix_acl_alloc(num_pos_acl + (!mask ? 1:0));
	if (!acl)
		goto out_res;
	i=0; 
	printk("nfs4_acl_to_posix: i=%d\n",i);
	/* user owner */
	if (user_owner_acl) {
		SET_MASK_NFS4(user_owner_acl->mask);
		acl->a_entries[i].e_tag = ACL_USER_OBJ;
		acl->a_entries[i].e_perm = mode;
		acl->a_entries[i].e_id = user_owner_acl->id;
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
	} // if
	
        /* user acls */
        list_for_each(p, &user_list){ 
		tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
                tmp_acl = tmp_list->acl;
                SET_MASK_NFS4(tmp_acl->mask);
		acl->a_entries[i].e_tag = ACL_USER;
		acl->a_entries[i].e_perm = mode;
		acl->a_entries[i].e_id = tmp_acl->id;
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
		if (mask != -1)
			mask |= mode;
        } // list_for_each

	if (group_owner_acl) {
		SET_MASK_NFS4(group_owner_acl->mask);
		acl->a_entries[i].e_tag = ACL_GROUP_OBJ;
		acl->a_entries[i].e_perm = mode;
		acl->a_entries[i].e_id = group_owner_acl->id;
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
		if (mask != -1)
			mask |= mode;
	} // if

        /* group acls */
        list_for_each(p, &group_list){ 
		tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
                tmp_acl = tmp_list->acl;
                SET_MASK_NFS4(tmp_acl->mask);
		acl->a_entries[i].e_tag = ACL_GROUP;
		acl->a_entries[i].e_perm = mode;
		acl->a_entries[i].e_id = tmp_acl->id;
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
		if (mask != -1)
			mask |= mode;
        } // list_for_each
	
	/* mask */
	if (mask != -1) {
		acl->a_entries[i].e_tag = ACL_MASK;
		acl->a_entries[i].e_perm = mask;
		acl->a_entries[i].e_id = -1;
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
	} // if

	/* other */
        list_for_each(p, &other_list){ 
		tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
                tmp_acl = tmp_list->acl;
                SET_MASK_NFS4(tmp_acl->mask);
		acl->a_entries[i].e_tag = ACL_OTHER;
		acl->a_entries[i].e_perm = mode;
		acl->a_entries[i].e_id = tmp_acl->id; 
		i++;
		printk("nfs4_acl_to_posix: i=%d\n",i);
	} // list_for_each
	printk("nfsv4_acl_to_posix_acl: i(no acls)=%d\n", i);

	printk("nfsv4_acl_to_posix_acl: cleanup time\n");
        /* cleanup */

        while(!list_empty(&user_list)) {
		printk("user_list free\n");
	        p = user_list.next;
	        list_del(p);
	        tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
	        NFS4_FREE(tmp_list);
	} // while              

        while(!list_empty(&group_list)) {
                printk("group_list free\n");
                p = group_list.next;
                list_del(p);
                tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
	        NFS4_FREE(tmp_list);
	} // while              

	while(!list_empty(&other_list)) {
	        printk("other_list free \n");
	        p = other_list.next;
	        list_del(p);
	        tmp_list = list_entry(p, nfsv4_acl_list, l_acl);
	        NFS4_FREE(tmp_list);
	} // while         
	printk("nfsv4_acl_to_posix_acl: I'm out\n");
out_res:
	return acl;
}


#endif // #ifdef NFS4ACL



