/*
 *  linux/fs/nfs4/nfs4_syms.c
 *
 *  Copyright (c) 2000 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#define __NO_VERSION__
#include <linux/config.h>
#include <linux/module.h>

#include <linux/rpc.h>
#include <linux/nfs4/nfs4_prot.h>
#include <linux/nfs4/nfs4_debug.h>
#include <linux/nfs4/nfs4.h>

#if NFS4_DEBUG
/* Debugging flags */
EXPORT_SYMBOL(nfsv4_debug_level1);
EXPORT_SYMBOL(nfsv4_debug_level2);
EXPORT_SYMBOL(nfsv4_debug_level3);
EXPORT_SYMBOL(nfsv4_debug_mount);
EXPORT_SYMBOL(nfsv4_debug_ctl);
EXPORT_SYMBOL(nfsv4_debug_dispatcher);
EXPORT_SYMBOL(nfsv4_debug_fh);
EXPORT_SYMBOL(nfsv4_debug_xdr);
EXPORT_SYMBOL(nfsv4_debug_getattr);
EXPORT_SYMBOL(nfsv4_debug_setattr);
EXPORT_SYMBOL(nfsv4_debug_lookup);
EXPORT_SYMBOL(nfsv4_debug_readdir);
EXPORT_SYMBOL(nfsv4_debug_create);
EXPORT_SYMBOL(nfsv4_debug_remove);
EXPORT_SYMBOL(nfsv4_debug_rename);
EXPORT_SYMBOL(nfsv4_debug_link);
EXPORT_SYMBOL(nfsv4_debug_readlink);
EXPORT_SYMBOL(nfsv4_debug_open);
EXPORT_SYMBOL(nfsv4_debug_read);
EXPORT_SYMBOL(nfsv4_debug_write);
EXPORT_SYMBOL(nfsv4_debug_lock);
EXPORT_SYMBOL(nfsv4_debug_seqid);
EXPORT_SYMBOL(nfsv4_debug_stateid);
EXPORT_SYMBOL(nfsv4_debug_lockowner);
EXPORT_SYMBOL(nfsv4_debug_clientid);
EXPORT_SYMBOL(nfsv4_debug_reval);
EXPORT_SYMBOL(nfsv4_debug_auth);
EXPORT_SYMBOL(nfsv4_debug_secinfo);
EXPORT_SYMBOL(nfsv4_debug_pagecache);
EXPORT_SYMBOL(nfsv4_debug_lease);
EXPORT_SYMBOL(nfsv4_debug_async);
#ifdef NFS4ACL
EXPORT_SYMBOL(nfsv4_debug_acl);
#endif /* NFS4ACL */

/* Functions for debugging vnode refcounts, memory allocation */
EXPORT_SYMBOL(__NFS4_ALLOC);
EXPORT_SYMBOL(__NFS4_FREE);
EXPORT_SYMBOL(__IGET);
EXPORT_SYMBOL(__IPUT);
EXPORT_SYMBOL(__IGRAB);
EXPORT_SYMBOL(__DGET);
EXPORT_SYMBOL(__DGET_LOCKED);
EXPORT_SYMBOL(__D_ALLOC);
EXPORT_SYMBOL(__D_ALLOC_ROOT);
EXPORT_SYMBOL(__DPUT);
EXPORT_SYMBOL(__D_LOOKUP);
EXPORT_SYMBOL(__LOOKUP_ONE);
EXPORT_SYMBOL(__FOLLOW_DOWN);
#endif  /* NFS4_DEBUG */

/* Utilities */
EXPORT_SYMBOL(kernerrno);
EXPORT_SYMBOL(nfserrno);
EXPORT_SYMBOL(utf8dup);
EXPORT_SYMBOL(utf8cmp);
EXPORT_SYMBOL(ip_to_clientaddr4);

/* Debugging print statements -- should be moved within #if NFS4_DEBUG eventually */
EXPORT_SYMBOL(hex_dump);
EXPORT_SYMBOL(print_bm);
EXPORT_SYMBOL(print_fattr);
EXPORT_SYMBOL(print_lockowner);
EXPORT_SYMBOL(print_pathname);
EXPORT_SYMBOL(print_fh);
EXPORT_SYMBOL(print_rpcsec_gss_info);

/* GSS */
EXPORT_SYMBOL(gss_init);
EXPORT_SYMBOL(gss_get_name);
EXPORT_SYMBOL(gss_get_num);
EXPORT_SYMBOL(gss_free);
EXPORT_SYMBOL(gss_shutdown);

/* XDR */
EXPORT_SYMBOL(xdrclnt_create);
EXPORT_SYMBOL(xdrsvc_create);
EXPORT_SYMBOL(xdr_COMPOUND4args);
EXPORT_SYMBOL(xdr_COMPOUND4res);

/* acl */
#ifdef NFS4ACL
EXPORT_SYMBOL(posix_acl_to_nfsv4_acl);
EXPORT_SYMBOL(nfsv4_acl_to_posix_acl);
#endif
