/*
 * sysctl.c
 *
 *  Copyright (c) 2000 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Jake Moilanen <jmoilane@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <linux/sysctl.h>
#include <linux/fs.h>
#include <linux/nfs4/nfs4_debug.h>

int nfsv4_debug_level1          = 0;
int nfsv4_debug_level2          = 0;
int nfsv4_debug_level3          = 0;

int nfsv4_debug_mount           = 0;
int nfsv4_debug_ctl             = 0;
int nfsv4_debug_dispatcher      = 0;
int nfsv4_debug_fh              = 0;
int nfsv4_debug_xdr             = 0;
int nfsv4_debug_gss             = 0;

int nfsv4_debug_getattr         = 0;
int nfsv4_debug_setattr         = 0;
int nfsv4_debug_lookup          = 0;
int nfsv4_debug_readdir         = 0;
int nfsv4_debug_create          = 0;
int nfsv4_debug_remove          = 0;
int nfsv4_debug_rename          = 0;
int nfsv4_debug_link            = 0;
int nfsv4_debug_readlink        = 0;
int nfsv4_debug_open            = 0;
int nfsv4_debug_read            = 0;
int nfsv4_debug_write           = 0;
int nfsv4_debug_lock            = 0;

int nfsv4_debug_seqid           = 0;   /* XXX: phase this out in favor of debug_lockowner */
int nfsv4_debug_stateid         = 0;
int nfsv4_debug_lockowner       = 0;
int nfsv4_debug_clientid        = 0;
int nfsv4_debug_reval           = 0;
int nfsv4_debug_auth            = 0;
int nfsv4_debug_secinfo         = 0;
int nfsv4_debug_pagecache       = 0;    /* including mmap() */
int nfsv4_debug_lease           = 0;
int nfsv4_debug_async		= 0;
int nfsv4_debug_check           = 0;
int nfsv4_debug_acl		= 0;


#if NFS4_DEBUG

static struct ctl_table_header *nfs4_table_header;
static ctl_table                nfs4_table[];

void
rpc_register_sysctl(void)
{
	if (!nfs4_table_header)
		nfs4_table_header = register_sysctl_table(nfs4_table, 1);
}

void
rpc_unregister_sysctl(void)
{
	if (nfs4_table_header) {
		unregister_sysctl_table(nfs4_table_header);
		nfs4_table_header = NULL;
	}
}

#define DIRENTRY(nam1, nam2, child)     \
        {CTL_##nam1, #nam2, NULL, 0, 0555, child }
#define DBGENTRY(nam1, nam2)    \
        {CTL_DEBUG_##nam1, #nam2, &nfsv4_debug_##nam2, sizeof(int),\
         0644, NULL, &proc_dointvec}

static ctl_table              debug_table[] = {
	DBGENTRY(LEVEL1, level1),
	DBGENTRY(LEVEL2, level2),
	DBGENTRY(LEVEL3, level3),
	DBGENTRY(MOUNT, mount),
	DBGENTRY(CTL, ctl),
	DBGENTRY(DISPATCHER, dispatcher),
	DBGENTRY(FH, fh),
	DBGENTRY(XDR, xdr),
	DBGENTRY(GSS, gss),
	DBGENTRY(GETATTR, getattr),
	DBGENTRY(SETATTR, setattr),
	DBGENTRY(LOOKUP, lookup),
	DBGENTRY(READDIR, readdir),
	DBGENTRY(CREATE, create),
	DBGENTRY(REMOVE, remove),
	DBGENTRY(RENAME, rename),
	DBGENTRY(LINK, link),
	DBGENTRY(READLINK, readlink),
	DBGENTRY(OPEN, open),
	DBGENTRY(READ, read),
	DBGENTRY(WRITE, write),
	DBGENTRY(LOCK, lock),
	DBGENTRY(SEQID, seqid),
	DBGENTRY(STATEID, stateid),
	DBGENTRY(LOCKOWNER, lockowner),
	DBGENTRY(CLIENTID, clientid),
	DBGENTRY(REVAL, reval),
	DBGENTRY(AUTH, auth),
	DBGENTRY(SECINFO, secinfo),
	DBGENTRY(PAGECACHE, pagecache),
	DBGENTRY(LEASE, lease),
	DBGENTRY(ASYNC, async),
	DBGENTRY(CHECK, check),
	DBGENTRY(ACL, acl),
	{0}
};

static ctl_table             nfs4_table[] = {
        DIRENTRY(NFS4, nfs4, debug_table),
        {0}
};


#endif  /* NFS4_DEBUG */
