/*
  File: linux/acl_ea.h

  (extended attribute representation of access control lists)

  (C) 2000 Andreas Gruenbacher, <a.gruenbacher@computer.org>
*/

#include <linux/posix_acl.h>

#define ACL_EA_ACCESS	"$acl"
#define ACL_EA_DEFAULT	"$defacl"

#define ACL_EA_VERSION	0x0001

#define ACL_EA_MAX_ENTRIES	32

typedef struct {
	__u16		e_tag;
	__u16		e_perm;
	__u32		e_id;
} acl_ea_entry;

typedef struct {
	__u16		e_tag;
	__u16		e_perm;
} acl_ea_entry_short;

typedef struct {
	__u32		a_version;
} acl_ea_header;

static inline size_t acl_ea_size(int count)
{
	if (count <= 4) {
		return sizeof(acl_ea_header) +
		       count * sizeof(acl_ea_entry_short);
	} else {
		return sizeof(acl_ea_header) +
		       4 * sizeof(acl_ea_entry_short) +
		       (count - 4) * sizeof(acl_ea_entry);
	}
}

static inline int acl_ea_count(size_t size)
{
	ssize_t s;
	size -= sizeof(acl_ea_header);
	s = size - 4 * sizeof(acl_ea_entry_short);
	if (s < 0) {
		if (size % sizeof(acl_ea_entry_short))
			return -1;
		return size / sizeof(acl_ea_entry_short);
	} else {
		if (s % sizeof(acl_ea_entry))
			return -1;
		return s / sizeof(acl_ea_entry) + 4;
	}
}

#ifdef __KERNEL__

posix_acl_t *
acl_ea_get_inode(
	struct inode *inode,
	int type);
int
acl_ea_set_inode(
	struct inode *inode,
	int type,
	posix_acl_t *acl);

#endif

