/*
  File: linux/ext_attr.h

  Extended attributes handling.

  (C) 2000 Andreas Gruenbacher, <a.gruenbacher@computer.org>
*/

/* Maximum length of an extended attribute name (arbitrary limit) */
#define EXT_ATTR_NAME_MAX	32

/* Maximum size of an extended attribute value (arbitrary limit) */
#define EXT_ATTR_VALUE_MAX	4096

/* Operations */
#define EXT_ATTR_GET		1
#define EXT_ATTR_GET_SIZE	2
#define EXT_ATTR_SET		3

#ifdef __KERNEL__

/* Definition of a system attribute */
struct ext_attr_handler {
	struct list_head	link;
	const char		*name;	/* name begins with a '$'. */
	int (*get_inode) (struct inode *, const char *, void *, size_t, int);
	int (*set_inode) (struct inode *, const char *, void *, size_t, int);
};

extern int register_ext_attr_handler(struct ext_attr_handler *);
extern void unregister_ext_attr_handler(struct ext_attr_handler *);

/* Extended attribute manipulation */
extern int ext_attr_get(struct inode *, const char *, void *, size_t, int);
extern int ext_attr_set(struct inode *, const char *, void *, size_t, int);

#define EXT_ATTR_FLAG_EXISTS	(1<<16)  /* attribute must exist already */
#define EXT_ATTR_FLAG_CREATE	(1<<17)  /* attribute must not exist already */
#define EXT_ATTR_FLAG_USER	(1<<18)  /* copy to/from userspace */

#else  /* __KERNEL__ */

int
ext_attr_path(
        const char *path,
	int cmd,
	const char *name,
	void *value,
	size_t size);
int
ext_attr_fd(
        int fd,
	int cmd,
	const char *name,
	void *value,
	size_t size);

#endif /* __KERNEL__ */
