/*
 *  linux/fs/nfs4/nfs4.h
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _LINUX_NFS4_NFS4_H
#define _LINUX_NFS4_NFS4_H

#define NFS4_PORT			2049
//#define NFS4ACL				1

static inline u32
opaque_hashval(const void *ptr, int nbytes)
{
	unsigned char *cptr = (unsigned char *) ptr;
	u32 x = 0;

	while (nbytes--) {
		x *= 37;
		x += *cptr++;
	}
	return x;
}

/* nfs4util.c */
extern int kernerrno(nfsstat4 err);
extern int nfserrno(int errno);
extern int utf8dup(utf8string *dest, utf8string *src);
extern int utf8cmp(utf8string *s1, utf8string *s2);
extern void ip_to_clientaddr4(u32 ip_addr, clientaddr4 *cp);
extern void hex_dump(void *ptr, int nbytes);
extern void print_bm(bitmap4 *bm);
extern void print_fattr(fattr4 *fp);
extern void print_lockowner(nfs_lockowner4 *lp);
extern void print_pathname(pathname4 *p);
extern void print_fh(nfs_fh4 *fh);
extern void print_rpcsec_gss_info(rpcsec_gss_info *ptr);

/* nfs4gss.c */
/* XXX : the gss stuff assumes uid_t == gid_t; is this ok?  (The same assumption is made in
   in the definition of 'struct nfs4fs_fattr' in fs/nfs4fs/decode.c) */
#define GSS_OWNER              0
#define GSS_GROUP              1
#define GSS_CACHE_INLINE_LEN  32
#define GSS_NAME_MAX         256

struct gss_cacheent {
	uid_t id;
	struct qstr name;

	/* fields after this point are private, for use by the gss cache - don't use */
	atomic_t refcount;
	struct list_head name_hash;
	struct list_head id_hash;
};

extern void gss_init(void);
extern int gss_get_name(int type, uid_t id, struct gss_cacheent **cachep);
extern int gss_get_num(int type, unsigned int len, const unsigned char *name, uid_t *idp);
extern void gss_free(struct gss_cacheent *p);
extern void gss_shutdown(void);

static inline void
gss_put(struct gss_cacheent *p)
{
	if (atomic_dec_and_test(&p->refcount))
		gss_free(p);
}
/* call interface */
int get_num (u_int type, utf8string *str, u_int *num);
int get_name (u_int type, u_int num, utf8string *str);

/* Test whether start+len >= 2^64, written in a way that doesn't make assumptions about how
   the underlying hardware deals with arithmetic overflow. */
#define LOFF_OVERFLOW(start, len)      ((u64)(len) > ~(u64)(start))

#endif

/* acl.c */
#ifdef NFS4ACL
fattr4_acl * posix_acl_to_nfsv4_acl(posix_acl_t *);
posix_acl_t * nfsv4_acl_to_posix_acl(nfs4_acl *, int);
#endif
