/*
 *  linux/fs/nfs4/nfs4_debug.h
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _LINUX_NFS4_NFS4_DEBUG_H
#define _LINUX_NFS4_NFS4_DEBUG_H

#define NFS4_DEBUG       1
#define RPCDBG_FACILITY  0

/* XXX : do memory alloction with GFP_NFS instead of GFP_KERNEL?? */

#if NFS4_DEBUG
# define nfsv4_printk(fac,args...)   do { if (nfsv4_debug_##fac) printk(args); } while (0)
# define nfsv4_hexdump(fac,ptr,len)  do { if (nfsv4_debug_##fac) hex_dump(ptr,len); } while (0)
# define nfsv4_print_rpcsec_gss_info(fac,rpcsec_gss_info)      do { if (nfsv4_debug_##fac) print_rpcsec_gss_info(rpcsec_gss_info); } while (0)
# define nfsv4_print_fh(fac,fh)      do { if (nfsv4_debug_##fac) print_fh(fh); } while (0)
# define nfsv4_print_svcfh(fac,fh)   do { if (nfsv4_debug_##fac) print_svcfh(fh); } while (0)
# define nfsv4_print_bm(fac,bm)      do { if (nfsv4_debug_##fac) print_bm(bm); } while (0)
# define nfsv4_print_fattr(fac,fp)   do { if (nfsv4_debug_##fac) print_fattr(fp); } while (0)
# define nfsv4_print_lockowner(fac,lp)  do { if (nfsv4_debug_##fac) \
	print_lockowner(lp); } while (0)
# define nfsv4_print_pathname(fac,p)    do { if (nfsv4_debug_##fac) \
	print_pathname(p); } while (0)
# define NFS4_ALLOC(nbytes)	__NFS4_ALLOC(nbytes, __FILE__, __LINE__)
# define NFS4_FREE(ptr)		__NFS4_FREE(ptr, __FILE__, __LINE__)
# define IGET(s, i) __IGET(s, i, __FILE__, __LINE__)
# define IPUT(i) __IPUT(i, __FILE__, __LINE__)
# define IGRAB(i) __IGRAB(i, __FILE__, __LINE__)
# define DGET(d) __DGET(d, __FILE__, __LINE__)
# define DGET_LOCKED(d) __DGET_LOCKED(d, __FILE__, __LINE__)
# define D_ALLOC(d, q) __D_ALLOC(d, q, __FILE__, __LINE__)
# define D_ALLOC_ROOT(i) __D_ALLOC_ROOT(i, __FILE__, __LINE__)
# define DPUT(d) __DPUT(d, __FILE__, __LINE__)
# define D_LOOKUP(d, q) __D_LOOKUP(d, q, __FILE__, __LINE__)
# define LOOKUP_ONE(n, d) __LOOKUP_ONE(n, d, __FILE__, __LINE__)
# define FOLLOW_DOWN(m, d) __FOLLOW_DOWN(m, d, __FILE__, __LINE__)
#else
# define nfsv4_printk(fac,args...)	do { } while (0)
# define nfsv4_hexdump(fac,ptr,len)	do { } while (0)
# define nfsv4_print_fh(fac,fh)		do { } while (0)
# define nfsv4_print_svcfh(fac,fh)	do { } while (0)
# define nfsv4_print_bm(fac,bm)		do { } while (0)
# define nfsv4_print_fattr(fac,fp)	do { } while (0)
# define nfsv4_print_lockowner(fac,lp)	do { } while (0)
# define nfsv4_print_pathname(fac,p)	do { } while (0)
# define NFS4_ALLOC(nbytes)	kmalloc(nbytes, GFP_KERNEL)
# define NFS4_FREE(ptr)		kfree(ptr)
# define IGET(s, i)		iget(s, i)
# define IPUT(i)		iput(i)
# define IGRAB(i)		igrab(i)
# define DGET(d)		dget(d)
# define DGET_LOCKED(d)		dget_locked(d)
# define D_ALLOC(d, q)		d_alloc(d, q)
# define D_ALLOC_ROOT(i)	d_alloc_root(i)
# define DPUT(d)		dput(d)
# define D_LOOKUP(d, q)		d_lookup(d, q)
# define LOOKUP_ONE(n, d)	lookup_one(n, d)
# define FOLLOW_DOWN(m, d)	follow_down(m, d)
#endif

#if NFS4_DEBUG

extern void *__NFS4_ALLOC(int nbytes, char *file, int line);
extern void __NFS4_FREE(void *ptr, char *file, int line);
extern void NFS4_COMPLAIN(void);
extern struct inode *__IGET(struct super_block *sb, unsigned long ino, char *file, int line);
extern void __IPUT(struct inode *inode, char *file, int line);
extern struct inode * __IGRAB(struct inode *inode, char *file, int line);
extern struct dentry *__DGET(struct dentry *dentry, char *file, int line);
extern struct dentry *__DGET_LOCKED(struct dentry *dentry, char *file, int line);
extern struct dentry *__D_ALLOC(struct dentry *dentry, const struct qstr *str,
				char *file, int line);
extern struct dentry *__D_ALLOC_ROOT(struct inode *inode, char *file, int line);
extern void __DPUT(struct dentry *dentry, char *file, int line);
extern struct dentry *__D_LOOKUP(struct dentry *dentry, struct qstr *str,
				 char *file, int line);
extern struct dentry *__LOOKUP_ONE(const char *str, struct dentry *dentry,
				   char *file, int line);
extern int __FOLLOW_DOWN(struct vfsmount **, struct dentry **, char *file, int line);

extern int nfsv4_debug_level1;
extern int nfsv4_debug_level2;
extern int nfsv4_debug_level3;
extern int nfsv4_debug_mount;
extern int nfsv4_debug_ctl;
extern int nfsv4_debug_dispatcher;
extern int nfsv4_debug_fh;
extern int nfsv4_debug_getattr;
extern int nfsv4_debug_setattr;
extern int nfsv4_debug_lookup;
extern int nfsv4_debug_readdir;
extern int nfsv4_debug_create;
extern int nfsv4_debug_xdr;
extern int nfsv4_debug_gss;
extern int nfsv4_debug_remove;
extern int nfsv4_debug_link;
extern int nfsv4_debug_readlink;   /* change name to nfsv4_debug_symlink at some point */
extern int nfsv4_debug_rename;
extern int nfsv4_debug_open;
extern int nfsv4_debug_read;
extern int nfsv4_debug_write;
extern int nfsv4_debug_lock;
extern int nfsv4_debug_seqid;
extern int nfsv4_debug_lockowner;
extern int nfsv4_debug_stateid;
extern int nfsv4_debug_clientid;
extern int nfsv4_debug_reval;
extern int nfsv4_debug_auth;
extern int nfsv4_debug_secinfo;
extern int nfsv4_debug_pagecache;
extern int nfsv4_debug_lease;
extern int nfsv4_debug_async;
extern int nfsv4_debug_check;
extern int nfsv4_debug_acl;

#define nfsv4_debug_ctl2		(nfsv4_debug_ctl || nfsv4_debug_level2)
#define nfsv4_debug_mount2		(nfsv4_debug_mount || nfsv4_debug_level2)
#define nfsv4_debug_fh2			(nfsv4_debug_fh || nfsv4_debug_level2)
#define nfsv4_debug_getattr2		(nfsv4_debug_getattr || nfsv4_debug_level2)
#define nfsv4_debug_setattr2		(nfsv4_debug_setattr || nfsv4_debug_level2)
#define nfsv4_debug_readdir2		(nfsv4_debug_readdir || nfsv4_debug_level2)
#define nfsv4_debug_create2		(nfsv4_debug_create || nfsv4_debug_level2)
#define nfsv4_debug_xdr2		(nfsv4_debug_xdr || nfsv4_debug_level2)
#define nfsv4_debug_lookup2		(nfsv4_debug_lookup || nfsv4_debug_level2)
#define nfsv4_debug_remove2		(nfsv4_debug_remove || nfsv4_debug_level2)
#define nfsv4_debug_link2		(nfsv4_debug_link || nfsv4_debug_level2)
#define nfsv4_debug_readlink2		(nfsv4_debug_readlink || nfsv4_debug_level2)
#define nfsv4_debug_rename2		(nfsv4_debug_rename || nfsv4_debug_level2)
#define nfsv4_debug_open2		(nfsv4_debug_open || nfsv4_debug_level2)
#define nfsv4_debug_read2		(nfsv4_debug_read || nfsv4_debug_level2)
#define nfsv4_debug_write2		(nfsv4_debug_write || nfsv4_debug_level2)
#define nfsv4_debug_lock2		(nfsv4_debug_lock || nfsv4_debug_level2)
#define nfsv4_debug_seqid2		(nfsv4_debug_seqid || nfsv4_debug_level2)
#define nfsv4_debug_lockowner2		(nfsv4_debug_lockowner || nfsv4_debug_level2)
#define nfsv4_debug_stateid2		(nfsv4_debug_stateid || nfsv4_debug_level2)
#define nfsv4_debug_clientid2		(nfsv4_debug_clientid || nfsv4_debug_level2)
#define nfsv4_debug_reval2		(nfsv4_debug_reval || nfsv4_debug_level2)
#define nfsv4_debug_gss2		(nfsv4_debug_gss || nfsv4_debug_level2)
#define nfsv4_debug_async2		(nfsv4_debug_async || nfsv4_debug_level2)
#define nfsv4_debug_auth2		(nfsv4_debug_auth || nfsv4_debug_level2)
#define nfsv4_debug_lease2		(nfsv4_debug_lease || nfsv4_debug_level2)
#define nfsv4_debug_dispatcher2		(nfsv4_debug_dispatcher || nfsv4_debug_level2)
#define nfsv4_debug_acl2		(nfsv4_debug_acl || nfsv4_debug_level2)

/*
 * Sysctl interface for RPC debugging
 */
void            rpc_register_sysctl(void);
void            rpc_unregister_sysctl(void);


/*
 * Declarations for the sysctl debug interface, which allows to read or
 * change the debug flags.
 */

#define CTL_NFS4      4739    /* arbitrary and hopefully unused */

enum {
  CTL_DEBUG_LEVEL1 = 1,
  CTL_DEBUG_LEVEL2,
  CTL_DEBUG_LEVEL3,
  CTL_DEBUG_MOUNT,
  CTL_DEBUG_CTL,
  CTL_DEBUG_DISPATCHER,
  CTL_DEBUG_FH,
  CTL_DEBUG_GETATTR,
  CTL_DEBUG_SETATTR,
  CTL_DEBUG_LOOKUP,
  CTL_DEBUG_READDIR,
  CTL_DEBUG_CREATE,
  CTL_DEBUG_XDR,
  CTL_DEBUG_GSS,
  CTL_DEBUG_REMOVE,
  CTL_DEBUG_LINK,
  CTL_DEBUG_READLINK,
  CTL_DEBUG_RENAME,
  CTL_DEBUG_OPEN,
  CTL_DEBUG_READ,
  CTL_DEBUG_WRITE,
  CTL_DEBUG_LOCK,
  CTL_DEBUG_SEQID,
  CTL_DEBUG_LOCKOWNER,
  CTL_DEBUG_STATEID,
  CTL_DEBUG_CLIENTID,
  CTL_DEBUG_REVAL,
  CTL_DEBUG_AUTH,
  CTL_DEBUG_SECINFO,
  CTL_DEBUG_PAGECACHE,
  CTL_DEBUG_LEASE,
  CTL_DEBUG_ASYNC,
  CTL_DEBUG_CHECK,
  CTL_DEBUG_ACL,
};

#endif  /* NFS4_DEBUG */

#endif






