/*
 *  linux/fs/nfs4/syscall.h
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 *  Syscall interface to nfsd
 */

#ifndef NFSD4_SYSCALL_H
#define NFSD4_SYSCALL_H

#include <asm/types.h>
#include <linux/posix_types.h>

/*
 * Version of the syscall interface -- increment whenever this file is modified!
 */
#define NFSCTL_VERSION		0x0208

/* Maximum number of allowed kernel threads */
#define	NFSD_MAXSERVS		128

/*
 * These are the commands understood by nfsctl().
 */
#define NFS4CTL_ADDPSEUDO	100
#define NFS4CTL_EXPORT		101
#define NFS4CTL_START		102
#define NFS4CTL_REFRESH		103
#define NFS4CTL_SETPUBFH	104

#define NFSD4_MAX_GRACE		300   /* seconds */

struct nfs4ctl_addpseudo {
	unsigned int	parent_id;
	unsigned int	namlen;
	char		*name;
};

struct nfs4ctl_export {
	unsigned int	id;
	unsigned int	pathlen;
	char		*path;
	unsigned int	read_only;
	
	/* Sec modes */
        unsigned int    sec_none;    /* AUTH_NONE RPC security */
        unsigned int    sec_sys;     /* AUTH_SYS RPC, default uid/gid lists */
        unsigned int    sec_dh;      /* AUTH_DES, old diffe-hellman encryption */
        unsigned int    sec_krb5;    /* RPCSEC_GSS Kerberos 5 authenication */
        unsigned int    sec_krb5i;   /* RPCSEC_GSS Kerberos 5 integrity */
        unsigned int    sec_krb5p;   /* RPCSEC_GSS Kerberso 5 protection */
        unsigned int    sec_spkm3;   /* SPKM authenication */
        unsigned int    sec_spkm3i;  /* SPKM integrity */
        unsigned int    sec_spkm3p;  /* SPKM protection */
        unsigned int    sec_lkey;    /* LIPKEY authenication */
        unsigned int    sec_lkeyi;   /* LIPKEY integrity */
        unsigned int    sec_lkeyp;   /* LIPKEY protection */

};

struct nfs4ctl_start {
	int		nthreads;
	unsigned int	grace_period;    /* boolean */
};

struct nfsctl_arg {
	int			ca_version;	/* safeguard */
	union {
		struct nfs4ctl_addpseudo	addpseudo;
		struct nfs4ctl_export		export;
		struct nfs4ctl_start		start;
		unsigned int			setpubfh;   /* export id of pubfh */
	} u;
};
#define ca_addpseudo	u.addpseudo
#define ca_export	u.export
#define ca_start	u.start
#define ca_setpubfh	u.setpubfh

union nfsctl_res {
	unsigned int	id;         /* used by ADDPSEUDO */
	unsigned int    nthreads;   /* used by REFRESH */
};

extern int nfsservctl(int cmd, struct nfsctl_arg *argp, union nfsctl_res *resp);

#endif /* NFSD4_SYSCALL_H */
