#ifndef _LINUX_NFS4_FS_I_H
#define _LINUX_NFS4_FS_I_H

/*
 *  linux/fs/nfs4_fs_i.h
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* XXX: currently, the flags field isn't used.. should we get rid of it? */

struct nfs4_inode_info {
	u64		fsid_major;
	u64		fsid_minor;
	u64		fileid;
	unsigned short	flags;
	unsigned long	cache_jiffies;    /* time we started caching this inode */
	u64		cache_changeid;   /* changeid at that time */
	unsigned long	cache_timeout;    /* timeout for cache revalidation*/
	unsigned long	cache_timeout_jiffies; /* time of last update to cache_timeout */
	struct nfs4fs_delegation   *delegation;
	int		snapshot;         /* do we have snapshot in page cache? (dir. only) */
	struct semaphore  locking_sema;   /* used for strict locking only */
	struct list_head  read_mappings;  /* for mmap() only */
	struct list_head  write_mappings; /* for mmap() only */

	/* used by flushd */
	unsigned long     flushd_nextscan;   /* wakeup time for this inode only */
	struct list_head  flushd_list;       /* linked into per-superblock list */
	struct list_head  flushd_read;       /* head of list of pending read requests */
	struct list_head  flushd_write;      /* pending write (= dirty+commit) requests */
	struct list_head  flushd_dirty;      /* pending dirty requests */
	struct list_head  flushd_commit;     /* pending commit requests */
	unsigned int      flushd_nread;      /* count of pending read requests */
	unsigned int      flushd_ndirty;     /* count of pending dirty requests */
	unsigned int      flushd_ncommit;    /* count of pending commit requests */
	unsigned int      flushd_dirtybytes; /* total size of pending dirty requests */
	
	/* used by recoverd */
	struct list_head  recovery_list;
};
#define NFS4_FSID_MAJOR(inode)			((inode)->u.nfs4_i.fsid_major)
#define NFS4_FSID_MINOR(inode)			((inode)->u.nfs4_i.fsid_minor)
#define NFS4_FILEID(inode)			((inode)->u.nfs4_i.fileid)
#define NFS4_FLAGS(inode)			((inode)->u.nfs4_i.flags)
#define NFS4_CACHE_JIFFIES(inode)		((inode)->u.nfs4_i.cache_jiffies)
#define NFS4_CACHE_CHANGEID(inode)		((inode)->u.nfs4_i.cache_changeid)
#define NFS4_CACHE_TIMEOUT(inode)		((inode)->u.nfs4_i.cache_timeout)
#define NFS4_CACHE_TIMEOUT_JIFFIES(inode)	((inode)->u.nfs4_i.cache_timeout_jiffies)
#define NFS4_DELEGATION(inode)                  ((inode)->u.nfs4_i.delegation)
#define NFS4_SNAPSHOT(inode)			((inode)->u.nfs4_i.snapshot)
#define NFS4_LOCKING_SEMAPHORE(inode)		((inode)->u.nfs4_i.locking_sema)
#define NFS4_READ_MAPPINGS(inode)		((inode)->u.nfs4_i.read_mappings)
#define NFS4_WRITE_MAPPINGS(inode)		((inode)->u.nfs4_i.write_mappings)
#define NFS4_FLUSHD_NEXTSCAN(inode)		((inode)->u.nfs4_i.flushd_nextscan)
#define NFS4_FLUSHD_LIST(inode)			((inode)->u.nfs4_i.flushd_list)
#define NFS4_FLUSHD_READ(inode)			((inode)->u.nfs4_i.flushd_read)
#define NFS4_FLUSHD_WRITE(inode)		((inode)->u.nfs4_i.flushd_write)
#define NFS4_FLUSHD_DIRTY(inode)		((inode)->u.nfs4_i.flushd_dirty)
#define NFS4_FLUSHD_COMMIT(inode)		((inode)->u.nfs4_i.flushd_commit)
#define NFS4_FLUSHD_NREAD(inode)		((inode)->u.nfs4_i.flushd_nread)
#define NFS4_FLUSHD_NDIRTY(inode)		((inode)->u.nfs4_i.flushd_ndirty)
#define NFS4_FLUSHD_NCOMMIT(inode)		((inode)->u.nfs4_i.flushd_ncommit)
#define NFS4_FLUSHD_DIRTYBYTES(inode)		((inode)->u.nfs4_i.flushd_dirtybytes)
#define NFS4_RECOVERY_LIST(inode)		((inode)->u.nfs4_i.recovery_list)

#define NFS4_RSIZE(inode)                       ((inode)->i_sb->u.nfs4_sb.rsize)
#define NFS4_WSIZE(inode)                       ((inode)->i_sb->u.nfs4_sb.wsize)
#define NFS4_DTSIZE(inode)                      ((inode)->i_sb->u.nfs4_sb.dtsize)

#define NFS4_NEED_REVAL(inode)      (!NFS4_DELEGATION(inode) && \
                                     (jiffies > NFS4_CACHE_JIFFIES(inode) + \
                                                NFS4_CACHE_TIMEOUT(inode)))
#define NFS4_UPDATE_TIMEOUT(inode)  (jiffies > NFS4_CACHE_TIMEOUT_JIFFIES(inode) + \
                                               NFS4_CACHE_TIMEOUT(inode))
#define NFS4_MINTIMEOUT(inode)      (S_ISDIR(inode->i_mode) ? inode->i_sb->u.nfs4_sb.acdirmin \
                                                            : inode->i_sb->u.nfs4_sb.acregmin)
#define NFS4_MAXTIMEOUT(inode)      (S_ISDIR(inode->i_mode) ? inode->i_sb->u.nfs4_sb.acdirmax \
                                                            : inode->i_sb->u.nfs4_sb.acregmax)

/*  These auxiliary lock fields are only used for nfsv4 "strict locking".
 *  Invariant: current_type is
 *    a write type   iff   (base_type is a write type || (nr_blockedwrite != 0)
 *    UNLOCK_LT      iff   (base_type == UNLOCK_LT) && (nr_blockedread == 0) &&
 *                         (nr_blockedwrite == 0)
 *    a read type    otherwise
 */
struct nfs4_lock_info {
	int base_type;
	int current_type;
	int nr_blockedread;
	int nr_blockedwrite;
};
#define NFS4_BASE_TYPE(file_lock)           ((file_lock)->fl_u.nfs4_fl.base_type)
#define NFS4_CURRENT_TYPE(file_lock)        ((file_lock)->fl_u.nfs4_fl.current_type)
#define NFS4_NR_BLOCKEDREAD(file_lock)      ((file_lock)->fl_u.nfs4_fl.nr_blockedread)
#define NFS4_NR_BLOCKEDWRITE(file_lock)     ((file_lock)->fl_u.nfs4_fl.nr_blockedwrite)

#endif
