#ifndef _LINUX_NFS4_FS_SB_H
#define _LINUX_NFS4_FS_SB_H

/*
 *  linux/fs/nfs4/nfs4_fs_sb.h
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Kendrick Smith <kmsmith@umich.edu>
 *  Andy Adamson <andros@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

struct nfs4_sb_info {
	/* XXX : all this stuff taken from v3, do we want it all? */
	struct rpc_clnt *	client;		/* RPC client handle */
	int			flags;		/* various flags */
	unsigned int		rsize;		/* read size */
	unsigned int		rpages;		/* read size (in pages) */
	unsigned int		wsize;		/* write size */
	unsigned int		wpages;		/* write size (in pages) */
	unsigned int		dtsize;		/* readdir size */
#if 0
	unsigned int		bsize;		/* server block size */
#endif
	unsigned int		acregmin;	/* attr cache timeouts */
	unsigned int		acregmax;
	unsigned int		acdirmin;
	unsigned int		acdirmax;
	unsigned int		namelen;
	char *			hostname;	/* remote hostname */

	int			have_clientid;
	u64			clientid;
	char *			mnt_path;	/* remote mount point, as one big string */
	int			plen;	/* length of pathname array for mountpoint */
	struct qstr *		pval;	/* components of pathname array for mountpoint */

	int			strict_locking;
	int			silly_rename;
	unsigned long		lock_mintimeout;
	unsigned long		lock_maxtimeout;

	unsigned int		lease_time;		/* in jiffies */
	pid_t			pidof_recoverd;		/* 0 if recoverd not running */
	wait_queue_head_t	grace_wait_queue;
	struct list_head	file_list;
	unsigned long		last_renewal;		/* in jiffies */
	struct nfs4_async_compound  *renewd;

	/* for flushd */
	struct rpc_task		*flushd;
	unsigned long		flushd_timeout;   /* wakeup time for entire superblock */
	struct list_head	flushd_inodes;    /* list of inodes with pending async requests */

	/* for callbacks */
	char			ip_addr[16];      /* our own IP address */
	unsigned short		callback_port;
};

#endif
