/*
  File: linux/posix_acl.h

  (C) 2000 Andreas Gruenbacher, <a.gruenbacher@computer.org>
*/


#define ACL_UNDEFINED_ID	((unsigned int)-1)

/* a_type field in acl_user_posix_entry_t */
#define ACL_TYPE_ACCESS		(0x8000)
#define ACL_TYPE_DEFAULT	(0x4000)

/* e_tag entry in posix_acl_entry_t */
#define ACL_USER_OBJ		(0x01)
#define ACL_USER		(0x02)
#define ACL_GROUP_OBJ		(0x04)
#define ACL_GROUP		(0x08)
#define ACL_MASK		(0x10)
#define ACL_OTHER		(0x20)

/* permissions in the e_perm field */
#define ACL_READ		(0x04)
#define ACL_WRITE		(0x02)
#define ACL_EXECUTE		(0x01)
//#define ACL_ADD		(0x08)
//#define ACL_DELETE		(0x10)

#ifdef __KERNEL__

typedef struct {
	short			e_tag;
	unsigned short		e_perm;
	unsigned int		e_id;
} posix_acl_entry_t;

struct posix_acl_tag {
	atomic_t		a_refcount;
	unsigned int		a_count;
	posix_acl_entry_t	a_entries[0];
};
#ifndef posix_acl_t
typedef struct posix_acl_tag posix_acl_t;
# define posix_acl_t posix_acl_t
#endif

#define FOREACH_ACL_ENTRY(pa, acl, pe) \
	for(pa=(acl)->a_entries, pe=pa+(acl)->a_count; pa<pe; pa++)

/* pxacl.c */

posix_acl_t *
posix_acl_alloc(
	int count);
posix_acl_t *
posix_acl_dup(
	posix_acl_t *acl);
posix_acl_t *
posix_acl_clone(
	const posix_acl_t *acl);
void
posix_acl_release(
	posix_acl_t *acl);
int
posix_acl_valid(
	const posix_acl_t *acl);
int
posix_acl_check_perm(
	struct inode *inode,
	const posix_acl_t *acl,
	int want);
posix_acl_t *
posix_acl_from_inode(
	struct inode *inode);
int
posix_acl_equiv_mode(
	const posix_acl_t *acl,
	mode_t *modep);
int
posix_acl_create_masq(
	posix_acl_t *acl,
	mode_t *modep);
int
posix_acl_chmod_masq(
	posix_acl_t *acl,
	mode_t mode);
int
posix_acl_masq_nfs_v2_mode(
	posix_acl_t *acl,
	mode_t *modep);
int
posix_acl_permission(
	struct inode *inode,
	int mask);
int
posix_acl_chmod(
	struct inode *inode);
int
posix_acl_init_cred(
	struct inode *inode,
	struct inode *dir,
	mode_t mode,
	int policy);
/*int
posix_acl_cmp(
	const posix_acl_t *acl1,
	const posix_acl_t *acl2);
unsigned int
posix_acl_hash(
	posix_acl_t *acl);*/

#endif  /* __KERNEL__ */
