#ifndef _RPC_H
#define _RPC_H

#ifdef __linux__
# include <linux/stddef.h>
# include <linux/types.h>
# include <linux/unistd.h>
# include <linux/malloc.h>
# include <asm/param.h>
# include <linux/sunrpc/xdr.h>
# include <linux/sunrpc/xprt.h>
# include <linux/sunrpc/clnt.h>
# include <linux/sunrpc/svc.h>
# define int32_t		int
# define uint32_t		unsigned int
# define int64_t		long long int
# define uint64_t		unsigned long long int
# define bool_t			unsigned int
# define enum_t			unsigned int
# define mem_alloc(bsize)	__NFS4_ALLOC(bsize, __FILE__, __LINE__)
# define mem_free(ptr, bsize)	__NFS4_FREE(ptr, __FILE__, __LINE__)
extern void *__NFS4_ALLOC(int, char *, int);
extern void __NFS4_FREE(void *, char *, int);
# define xdrsvc_decode_buf(xdrs)	(((struct svc_buf *)(xdrs)->x_private)->buf)
# define xdrsvc_encode_buf(xdrs)	(((struct svc_buf *)(xdrs)->x_private)->buf)
# define xdrsvc_encode_buflen(xdrs)	((xdrs)->x_handy)
# define xdrclnt_decode_buf(xdrs)	((xdrs)->x_base)
# define xdrclnt_decode_buflen(xdrs)	((xdrs)->x_handy)
extern void hex_dump(void *, int);
#endif

#ifdef __OpenBSD__
# define mem_alloc(bsize)	kmalloc(bsize, GFP_KERNEL)   /* surely not right! */
# define mem_free(ptr, bsize)	kfree(ptr)                   /* sorry, jim! */
#endif

#ifndef __P
# define __P(x) x
#endif

#ifndef FALSE
# define FALSE 0
#endif
#ifndef TRUE
# define TRUE 1
#endif

enum xdr_op {
	XDR_ENCODE=0,
	XDR_DECODE=1,
	XDR_FREE=2
};

#define BYTES_PER_XDR_UNIT	(4)
#define RNDUP(x)  ((((x) + BYTES_PER_XDR_UNIT - 1) / BYTES_PER_XDR_UNIT) \
		    * BYTES_PER_XDR_UNIT)

#ifndef XDR_QUADLEN
# define XDR_QUADLEN (((l) + 3) >> 2)
#endif

#define XDR_LEN(l)   ((((l) + 3) >> 2) << 2)

typedef struct __rpc_xdr {
	enum xdr_op	x_op;		/* operation; fast additional param */
	struct xdr_ops {
		/* get a long from underlying stream */
		bool_t	(*x_getlong) __P((struct __rpc_xdr *, long *));
		/* put a long to " */
		bool_t	(*x_putlong) __P((struct __rpc_xdr *, long *));
		/* get some bytes from " */
		bool_t	(*x_getbytes) __P((struct __rpc_xdr *, caddr_t, u_int));
		/* put some bytes to " */
		bool_t	(*x_putbytes) __P((struct __rpc_xdr *, caddr_t, u_int));
		/* returns bytes off from beginning */
		u_int	(*x_getpostn) __P((struct __rpc_xdr *));
		/* lets you reposition the stream */
		bool_t  (*x_setpostn) __P((struct __rpc_xdr *, u_int));
		/* buf quick ptr to buffered data */
		int32_t	*(*x_inline) __P((struct __rpc_xdr *, u_int));
		/* free privates of this xdr_stream */
		void	(*x_destroy) __P((struct __rpc_xdr *));
	} *x_ops;
	caddr_t 	x_public;	/* users' data */
	caddr_t		x_private;	/* pointer to private data */
	caddr_t 	x_base;		/* private used for position info */
	int		x_handy;	/* extra private word */
} XDR;

/*
 * A xdrproc_t exists for each data type which is to be encoded or decoded.
 *
 * The second argument to the xdrproc_t is a pointer to an opaque pointer.
 * The opaque pointer generally points to a structure of the data type
 * to be decoded.  If this pointer is 0, then the type routines should
 * allocate dynamic storage of the appropriate size and return it.
 *
 */
typedef	bool_t (*xdrproc_t) __P((XDR *, void *));

/*
 * Operations defined on a XDR handle
 *
 * XDR		*xdrs;
 * long		*longp;
 * caddr_t	 addr;
 * u_int	 len;
 * u_int	 pos;
 */
#define XDR_GETLONG(xdrs, longp)			\
	(*(xdrs)->x_ops->x_getlong)(xdrs, longp)
#define xdr_getlong(xdrs, longp)			\
	(*(xdrs)->x_ops->x_getlong)(xdrs, longp)

#define XDR_PUTLONG(xdrs, longp)			\
	(*(xdrs)->x_ops->x_putlong)(xdrs, longp)
#define xdr_putlong(xdrs, longp)			\
	(*(xdrs)->x_ops->x_putlong)(xdrs, longp)

#define XDR_GETBYTES(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)
#define xdr_getbytes(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_getbytes)(xdrs, addr, len)

#define XDR_PUTBYTES(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)
#define xdr_putbytes(xdrs, addr, len)			\
	(*(xdrs)->x_ops->x_putbytes)(xdrs, addr, len)

#define XDR_GETPOS(xdrs)				\
	(*(xdrs)->x_ops->x_getpostn)(xdrs)
#define xdr_getpos(xdrs)				\
	(*(xdrs)->x_ops->x_getpostn)(xdrs)

#define XDR_SETPOS(xdrs, pos)				\
	(*(xdrs)->x_ops->x_setpostn)(xdrs, pos)
#define xdr_setpos(xdrs, pos)				\
	(*(xdrs)->x_ops->x_setpostn)(xdrs, pos)

#define	XDR_INLINE(xdrs, len)				\
	(*(xdrs)->x_ops->x_inline)(xdrs, len)
#define	xdr_inline(xdrs, len)				\
	(*(xdrs)->x_ops->x_inline)(xdrs, len)

#define	XDR_DESTROY(xdrs)				\
	if ((xdrs)->x_ops->x_destroy) 			\
		(*(xdrs)->x_ops->x_destroy)(xdrs)
#define	xdr_destroy(xdrs)				\
	if ((xdrs)->x_ops->x_destroy) 			\
		(*(xdrs)->x_ops->x_destroy)(xdrs)

/*
 * Support struct for discriminated unions.
 * You create an array of xdrdiscrim structures, terminated with
 * a entry with a null procedure pointer.  The xdr_union routine gets
 * the discriminant value and then searches the array of structures
 * for a matching value.  If a match is found the associated xdr routine
 * is called to handle that part of the union.  If there is
 * no match, then a default routine may be called.
 * If there is no match and no default routine it is an error.
 */
#define NULL_xdrproc_t ((xdrproc_t)0)
struct xdr_discrim {
	int	value;
	xdrproc_t proc;
};

/*
 * In-line routines for fast encode/decode of primitve data types.
 * Caveat emptor: these use single memory cycles to get the
 * data from the underlying buffer, and will fail to operate
 * properly if the data is not aligned.  The standard way to use these
 * is to say:
 *	if ((buf = XDR_INLINE(xdrs, count)) == NULL)
 *		return (FALSE);
 *	<<< macro calls >>>
 * where ``count'' is the number of bytes of data occupied
 * by the primitive data types.
 *
 * N.B. and frozen for all time: each data type here uses 4 bytes
 * of external representation.
 */
#define IXDR_GET_LONG(buf)		((long)ntohl((u_long)*(buf)++))
#define IXDR_PUT_LONG(buf, v)		(*(buf)++ = (long)htonl((u_long)v))

#define IXDR_GET_BOOL(buf)		((bool_t)IXDR_GET_LONG(buf))
#define IXDR_GET_ENUM(buf, t)		((t)IXDR_GET_LONG(buf))
#define IXDR_GET_U_LONG(buf)		((u_long)IXDR_GET_LONG(buf))
#define IXDR_GET_SHORT(buf)		((short)IXDR_GET_LONG(buf))
#define IXDR_GET_U_SHORT(buf)		((u_short)IXDR_GET_LONG(buf))

#define IXDR_PUT_BOOL(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_ENUM(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_LONG(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_SHORT(buf, v)		IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_SHORT(buf, v)	IXDR_PUT_LONG((buf), ((long)(v)))

/*
 * These are the "generic" xdr routines.
 */
extern bool_t	xdr_void	__P((void));
extern bool_t	xdr_int		__P((XDR *, int *));
extern bool_t	xdr_u_int	__P((XDR *, u_int *));
extern bool_t	xdr_long	__P((XDR *, long *));
extern bool_t	xdr_u_long	__P((XDR *, u_long *));
extern bool_t	xdr_short	__P((XDR *, short *));
extern bool_t	xdr_u_short	__P((XDR *, u_short *));
extern bool_t	xdr_int16_t	__P((XDR *, int16_t *));
extern bool_t	xdr_u_int16_t	__P((XDR *, u_int16_t *));
extern bool_t	xdr_int32_t	__P((XDR *, int32_t *));
extern bool_t	xdr_u_int32_t	__P((XDR *, u_int32_t *));
extern bool_t	xdr_int64_t	__P((XDR *, int64_t *));
extern bool_t	xdr_u_int64_t	__P((XDR *, u_int64_t *));
extern bool_t	xdr_bool	__P((XDR *, bool_t *));
extern bool_t	xdr_enum	__P((XDR *, enum_t *));
extern bool_t	xdr_array	__P((XDR *, char **, u_int *, u_int, u_int, xdrproc_t));
extern bool_t	xdr_bytes	__P((XDR *, char **, u_int *, u_int));
extern bool_t	xdr_opaque	__P((XDR *, caddr_t, u_int));
extern bool_t	xdr_string	__P((XDR *, char **, u_int));
extern bool_t	xdr_union	__P((XDR *, enum_t *, char *, struct xdr_discrim *, xdrproc_t));
extern bool_t	xdr_char	__P((XDR *, char *));
extern bool_t	xdr_u_char	__P((XDR *, u_char *));
extern bool_t	xdr_vector	__P((XDR *, char *, u_int, u_int, xdrproc_t));
extern bool_t	xdr_float	__P((XDR *, float *));
extern bool_t	xdr_double	__P((XDR *, double *));
extern bool_t	xdr_reference	__P((XDR *, caddr_t *, u_int, xdrproc_t));
extern bool_t	xdr_pointer	__P((XDR *, caddr_t *, u_int, xdrproc_t));
extern bool_t	xdr_wrapstring	__P((XDR *, char **));
extern void	xdr_free 	__P((xdrproc_t, char *));

#define xdr_uint64_t xdr_u_int64_t
#define xdr_uint32_t xdr_u_int32_t

/*
 * Common opaque bytes objects used by many rpc protocols;
 * declared here due to commonality.
 */
#define MAX_NETOBJ_SZ 1024
struct netobj {
	u_int	n_len;
	char	*n_bytes;
};
typedef struct netobj netobj;
extern bool_t   xdr_netobj __P((XDR *, struct netobj *));

#ifdef __linux__
extern int xdrclnt_create __P((XDR *xdrs, struct rpc_rqst *req, u32 *p, enum xdr_op op));
extern void xdrsvc_create __P((XDR *xdrs, struct svc_rqst *rqstp, enum xdr_op op));
#endif

#endif /* !_RPC_H */
