/*
 * exportfs4.c 
 *
 *  Copyright (c) 2000 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Jake Moilanen <jmoilane@umich.edu>
 *  Kendrick Smith <kmsmith@umich.edu>
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "exportfs4.h"

int 
add_pseudo(char *name, int parent)
{
  struct nfsctl_arg arg;
  union nfsctl_res res;
  int result;

  arg.ca_version = NFSCTL_VERSION;
  arg.ca_addpseudo.parent_id = parent;
  arg.ca_addpseudo.namlen = strlen(name);
  arg.ca_addpseudo.name = name;
  
  result = nfsservctl(NFS4CTL_ADDPSEUDO, &arg, &res);
  if (result){
    parent = result;
    return result;
  }
  return res.id;
  
}

int
add_export(struct nfsctl_arg *arg)
{
  union nfsctl_res res; 
  struct stat s;
  int result;

  arg->ca_version = NFSCTL_VERSION;

  if (stat(arg->ca_export.path, &s)){
    perror("stat");
    exit(1);
  }
  result = nfsservctl(NFS4CTL_EXPORT, arg, &res);

  if (result){
    perror(arg->ca_export.path);
    exit(1);
  }
  
  return 0;
}

/*
 * reset_struct
 *
 * This is mainly to reset the permission flags like read only and 
 * the security flags.
 *
 */
void 
reset_struct(struct nfsctl_arg *arg)
{
  arg->ca_export.path = 0;
  arg->ca_export.id = 0; 
  arg->ca_export.pathlen = 0; 

  /* file permission flags */
  arg->ca_export.read_only = 0;

  /* Sec flags */
  arg->ca_export.sec_none = 0;
  arg->ca_export.sec_sys = 0; 
  arg->ca_export.sec_dh = 0;
  arg->ca_export.sec_krb5 = 0;
  arg->ca_export.sec_krb5i = 0;
  arg->ca_export.sec_krb5p = 0;
  arg->ca_export.sec_spkm3 = 0; 
  arg->ca_export.sec_spkm3i = 0;
  arg->ca_export.sec_spkm3p = 0;
  arg->ca_export.sec_lkey = 0;
  arg->ca_export.sec_lkeyi = 0;
  arg->ca_export.sec_lkeyp = 0;
}


void
start_server(struct nfsctl_arg *arg)
{
  union nfsctl_res res;

  arg->ca_version = NFSCTL_VERSION;
  arg->ca_start.nthreads = NFSD4_NTHREADS;

  if (nfsservctl(NFS4CTL_START, arg, &res)) {
    perror("export");
    exit(1);
  }

}

