%{
/*
 * exportfs4.l
 *
 *  Copyright (c) 2000 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Jake Moilanen <jmoilane@umich.edu>
 *  Kendrick Smith <kmsmith@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "exportfs4.h"
#include "y.tab.h"             /* Token codes from the parser */
int parent_id = 0;
char *last_element;
struct nfsctl_arg arg;
static int first_time = 1;
int lineno = 1;

%}

%s VPATH EPATH FPERM SECINFO

element ([A-Za-z0-9_.~\-]+)
path ([A-Za-z0-9_.~\-\/]+)
ro ([rR][oO])
sec ([sS][eE][cC]=)
none ([nN][oO][nN][eE])
sys ([sS][yY][sS])
dh ([dD][hH])
krb5 ([kK][rR][bB][5])
krb5i ([kK][rR][bB][5][iI])
krb5p ([kK][rR][bB][5][pP])
spkm3 ([sS][pP][kK][mM][3])
spkm3i ([sS][pP][kK][mM][3][iI])
spkm3p ([sS][pP][kK][mM][3][pP])
lkey ([lL][kK][eE][yY])
lkeyi ([lL][kK][eE][yY][iI])
lkeyp ([lL][kK][eE][yY][pP])
ws ([ \t\r]+)
nl (\n)

%%
%{
if (first_time){
  BEGIN VPATH;                   /* Start in VPATH state */
  first_time = 0;
}

%}

 /* Virtual Path */
<VPATH>{element}/{ws}   { BEGIN EPATH; 
                          parent_id = add_pseudo(yytext, parent_id); 
                          yylval.element = yytext; 
                          return END_ELEMENT; }
<VPATH>{element}        { parent_id = add_pseudo(yytext, parent_id);  
                          yylval.element = yytext; 
                          return ELEMENT; }
<VPATH>\/{ws}		 { BEGIN EPATH;
			  parent_id = 0;
                          yylval.element = yytext;
                          return END_ELEMENT; }
<VPATH>\/           

 /* Export Path */
<EPATH>{path}           { BEGIN FPERM;
                          arg.ca_export.id = parent_id; 
                          arg.ca_export.pathlen = yyleng; 
                          last_element = (char *) malloc(yyleng); 
                          strcpy(last_element, yytext);
                          arg.ca_export.path = last_element;
                          yylval.element = yytext; 
                          return PATH; }

 /* Security */
<SECINFO>:
<SECINFO>{none}         { arg.ca_export.sec_none = 1; return SEC; }
<SECINFO>{sys}          { arg.ca_export.sec_sys = 1; return SEC; }
<SECINFO>{dh}           { arg.ca_export.sec_dh = 1; return SEC; }
<SECINFO>{krb5i}        { arg.ca_export.sec_krb5i = 1; return SEC; }
<SECINFO>{krb5p}        { arg.ca_export.sec_krb5p = 1; return SEC; }
<SECINFO>{krb5}         { arg.ca_export.sec_krb5 = 1; return SEC; }
<SECINFO>{spkm3i}       { arg.ca_export.sec_spkm3i = 1; return SEC; }
<SECINFO>{spkm3p}       { arg.ca_export.sec_spkm3p = 1; return SEC; }
<SECINFO>{spkm3}        { arg.ca_export.sec_spkm3 = 1; return SEC; }
<SECINFO>{lkeyi}        { arg.ca_export.sec_lkeyi = 1; return SEC; } 
<SECINFO>{lkeyp}        { arg.ca_export.sec_lkeyp = 1; return SEC; }
<SECINFO>{lkey}         { arg.ca_export.sec_lkey = 1; return SEC; }

{sec}                   { BEGIN SECINFO; }

 /* Permissions */
<FPERM>{ro}             { arg.ca_export.read_only = 1; 
                          return RO; }

 /* End of line */
{nl}                    { BEGIN VPATH; 
                          parent_id = 0; 
                          lineno++; 
                          return NL; }

 /* Ignore whitespace */
[ \t\r]+                         

 /* Ignore comments */
#.*                   

 /* Anything else */
. ;                                   

%%

yyerror(const char *msg)
{
  fprintf(stderr, "%d: %s at '%s'\n", lineno, msg, yytext);
}
